@@ -31,7 +31,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 228195 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 320506 $")
 
 #include <ctype.h>
 #include <errno.h>
@@ -152,7 +152,7 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 228195 $")
 		</syntax>
 		<description>
 			<para>This application is used to listen to the audio from an Asterisk channel. This includes the audio 
-			coming in and "out of the channel being spied on. If the <literal>chanprefix</literal> parameter is specified,
+			coming in and out of the channel being spied on. If the <literal>chanprefix</literal> parameter is specified,
 			only channels beginning with this string will be spied upon.</para>
 			<para>While spying, the following actions may be performed:</para>
 			<para> - Dialing <literal>#</literal> cycles the volume level.</para>
@@ -345,6 +345,7 @@ struct chanspy_translation_helper {
 	struct ast_audiohook bridge_whisper_audiohook;
 	int fd;
 	int volfactor;
+	struct ast_flags flags;
 };
 
 static void *spy_alloc(struct ast_channel *chan, void *data)
@@ -370,7 +371,7 @@ static int spy_generate(struct ast_channel *chan, void *data, int len, int sampl
 		return -1;
 	}
 
-	if (ast_test_flag(&csth->spy_audiohook, OPTION_READONLY)) {
+	if (ast_test_flag(&csth->flags, OPTION_READONLY)) {
 		/* Option 'o' was set, so don't mix channel audio */
 		f = ast_audiohook_read_frame(&csth->spy_audiohook, samples, AST_AUDIOHOOK_DIRECTION_READ, AST_FORMAT_SLINEAR);
 	} else {
@@ -486,7 +487,7 @@ static int channel_spy(struct ast_channel *chan, struct chanspy_ds *spyee_chansp
 			spyer_name, name);
 
 	memset(&csth, 0, sizeof(csth));
-	ast_copy_flags(&csth.spy_audiohook, flags, AST_FLAGS_ALL);
+	ast_copy_flags(&csth.flags, flags, AST_FLAGS_ALL);
 
 	ast_audiohook_init(&csth.spy_audiohook, AST_AUDIOHOOK_TYPE_SPY, "ChanSpy");
 
@@ -937,6 +938,7 @@ static int common_exec(struct ast_channel *chan, struct ast_flags *flags,
 			}
 
 			if (!ienf) {
+				ast_channel_unlock(peer);
 				continue;
 			}
 
@@ -962,7 +964,7 @@ static int common_exec(struct ast_channel *chan, struct ast_flags *flags,
 				}
 				if (!ast_test_flag(flags, OPTION_NAME) || res < 0) {
 					if (!ast_test_flag(flags, OPTION_NOTECH)) {
-						if (ast_fileexists(peer_name, NULL, NULL) != -1) {
+						if (ast_fileexists(peer_name, NULL, NULL) > 0) {
 							res = ast_streamfile(chan, peer_name, chan->language);
 							if (!res) {
 								res = ast_waitstream(chan, "");
