@@ -30,21 +30,37 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 115076 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 184707 $")
 
 #include "asterisk/pbx.h"
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
+#include "asterisk/app.h"
+
+/*** DOCUMENTATION
+	<application name="DumpChan" language="en_US">
+		<synopsis>
+			Dump Info About The Calling Channel.
+		</synopsis>
+		<syntax>
+			<parameter name="level">
+				<para>Minimun verbose level</para>
+			</parameter>
+		</syntax>
+		<description>
+			<para>Displays information on channel and listing of all channel
+			variables. If <replaceable>level</replaceable> is specified, output is only
+			displayed when the verbose level is currently set to that number
+			or greater.</para>
+		</description>
+		<see-also>
+			<ref type="application">NoOp</ref>
+			<ref type="application">Verbose</ref>
+		</see-also>
+	</application>
+ ***/
 
 static char *app = "DumpChan";
-static char *synopsis = "Dump Info About The Calling Channel";
-static char *desc =
-	"   DumpChan([<min_verbose_level>])\n"
-	"Displays information on channel and listing of all channel\n"
-	"variables. If min_verbose_level is specified, output is only\n"
-	"displayed when the verbose level is currently set to that number\n"
-	"or greater. \n";
-
 
 static int serialize_showchan(struct ast_channel *c, char *buf, size_t size)
 {
@@ -133,7 +149,7 @@ static int serialize_showchan(struct ast_channel *c, char *buf, size_t size)
 
 static int dumpchan_exec(struct ast_channel *chan, void *data)
 {
-	struct ast_str *vars = ast_str_alloca(BUFSIZ * 4); /* XXX very large! */
+	struct ast_str *vars = ast_str_thread_get(&ast_str_thread_global_buf, 16);
 	char info[1024];
 	int level = 0;
 	static char *line = "================================================================================";
@@ -141,10 +157,11 @@ static int dumpchan_exec(struct ast_channel *chan, void *data)
 	if (!ast_strlen_zero(data))
 		level = atoi(data);
 
-	pbx_builtin_serialize_variables(chan, &vars);
-	serialize_showchan(chan, info, sizeof(info));
-	if (option_verbose >= level)
-		ast_verbose("\nDumping Info For Channel: %s:\n%s\nInfo:\n%s\nVariables:\n%s%s\n", chan->name, line, info, vars->str, line);
+	if (option_verbose >= level) {
+		serialize_showchan(chan, info, sizeof(info));
+		pbx_builtin_serialize_variables(chan, &vars);
+		ast_verbose("\nDumping Info For Channel: %s:\n%s\nInfo:\n%s\nVariables:\n%s%s\n", chan->name, line, info, ast_str_buffer(vars), line);
+	}
 
 	return 0;
 }
@@ -156,7 +173,7 @@ static int unload_module(void)
 
 static int load_module(void)
 {
-	return ast_register_application(app, dumpchan_exec, synopsis, desc);
+	return ast_register_application_xml(app, dumpchan_exec);
 }
 
 AST_MODULE_INFO_STANDARD(ASTERISK_GPL_KEY, "Dump Info About The Calling Channel");
