@@ -26,7 +26,7 @@
  
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 176627 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 281722 $")
 
 #include "asterisk/file.h"
 #include "asterisk/pbx.h"
@@ -220,7 +220,7 @@ static int readexten_exec(struct ast_channel *chan, void *data)
 			if (res < 1) {		/* timeout expired or hangup */
 				if (ast_check_hangup(chan)) {
 					status = "HANGUP";
-				} else {
+				} else if (x == 0) {
 					pbx_builtin_setvar_helper(chan, arglist.variable, "t");
 					status = "TIMEOUT";
 				}
