@@ -25,13 +25,12 @@
  */
 
 /*** MODULEINFO
-	<depend>generic_odbc</depend>
-	<depend>ltdl</depend>
+	<depend>res_odbc</depend>
  ***/
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 236850 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 283318 $")
 
 #include <sys/types.h>
 #include <time.h>
