@@ -40,7 +40,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 158135 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 300574 $")
 
 #include <sqlite.h>
 
@@ -87,24 +87,26 @@ static char sql_create_table[] = "CREATE TABLE cdr ("
 #endif
 ");";
 
+static void format_date(char *buffer, size_t length, struct timeval *when)
+{
+	struct ast_tm tm;
+
+	ast_localtime(when, &tm, NULL);
+	ast_strftime(buffer, length, DATE_FORMAT, &tm);
+}
+
 static int sqlite_log(struct ast_cdr *cdr)
 {
 	int res = 0;
 	char *zErr = 0;
-	struct ast_tm tm;
 	char startstr[80], answerstr[80], endstr[80];
 	int count;
 
 	ast_mutex_lock(&sqlite_lock);
 
-	ast_localtime(&cdr->start, &tm, NULL);
-	ast_strftime(startstr, sizeof(startstr), DATE_FORMAT, &tm);
-
-	ast_localtime(&cdr->answer, &tm, NULL);
-	ast_strftime(answerstr, sizeof(answerstr), DATE_FORMAT, &tm);
-
-	ast_localtime(&cdr->end, &tm, NULL);
-	ast_strftime(endstr, sizeof(endstr), DATE_FORMAT, &tm);
+	format_date(startstr, sizeof(startstr), &cdr->start);
+	format_date(answerstr, sizeof(answerstr), &cdr->answer);
+	format_date(endstr, sizeof(endstr), &cdr->end);
 
 	for(count=0; count<5; count++) {
 		res = sqlite_exec_printf(db,
@@ -161,9 +163,10 @@ static int sqlite_log(struct ast_cdr *cdr)
 
 static int unload_module(void)
 {
-	if (db)
-		sqlite_close(db);
 	ast_cdr_unregister(name);
+	if (db) {
+		sqlite_close(db);
+	}
 	return 0;
 }
 
@@ -173,8 +176,8 @@ static int load_module(void)
 	char fn[PATH_MAX];
 	int res;
 
-	ast_log(LOG_WARNING, "This module has been marked deprecated in favor of "
-		"using cdr_sqlite3_custom. (May be removed after Asterisk 1.6)\n");
+	ast_log(LOG_NOTICE, "This module has been marked deprecated in favor of "
+		"using cdr_sqlite3_custom.\n");
 
 	/* is the database there? */
 	snprintf(fn, sizeof(fn), "%s/cdr.db", ast_config_AST_LOG_DIR);
