@@ -62,7 +62,7 @@ CREATE TABLE [dbo].[cdr] (
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 159818 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 269010 $")
 
 #include <time.h>
 #include <math.h>
@@ -359,7 +359,7 @@ static int mssql_connect(void)
 		goto failed;
 	}
 
-	if (execute_and_consume(settings->dbproc, "SELECT 1 FROM [%s]", settings->table)) {
+	if (execute_and_consume(settings->dbproc, "SELECT 1 FROM [%s] WHERE 1 = 0", settings->table)) {
 		ast_log(LOG_ERROR, "Unable to find table '%s'\n", settings->table);
 		goto failed;
 	}
