@@ -25,7 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 150729 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 267507 $")
 
 #include "asterisk/module.h"
 #include "asterisk/config.h"
@@ -79,7 +79,6 @@ static struct ast_translator alawtolin = {
 	.sample = alaw_sample,
 	.buffer_samples = BUFFER_SAMPLES,
 	.buf_size = BUFFER_SAMPLES * 2,
-	.plc_samples = 160,
 };
 
 static struct ast_translator lintoalaw = {
@@ -92,29 +91,10 @@ static struct ast_translator lintoalaw = {
 	.buf_size = BUFFER_SAMPLES,
 };
 
-static int parse_config(int reload)
-{
-	struct ast_variable *var;
-	struct ast_flags config_flags = { reload ? CONFIG_FLAG_FILEUNCHANGED : 0 };
-	struct ast_config *cfg = ast_config_load("codecs.conf", config_flags);
-	if (cfg == CONFIG_STATUS_FILEMISSING || cfg == CONFIG_STATUS_FILEUNCHANGED || cfg == CONFIG_STATUS_FILEINVALID)
-		return 0;
-	for (var = ast_variable_browse(cfg, "plc"); var; var = var->next) {
-		if (!strcasecmp(var->name, "genericplc")) {
-			alawtolin.useplc = ast_true(var->value) ? 1 : 0;
-			ast_verb(3, "codec_alaw: %susing generic PLC\n", alawtolin.useplc ? "" : "not ");
-		}
-	}
-	ast_config_destroy(cfg);
-	return 0;
-}
-
 /*! \brief standard module stuff */
 
 static int reload(void)
 {
-	if (parse_config(1))
-		return AST_MODULE_LOAD_DECLINE;
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
@@ -132,8 +112,6 @@ static int load_module(void)
 {
 	int res;
 
-	if (parse_config(0))
-		return AST_MODULE_LOAD_DECLINE;
 	res = ast_register_translator(&alawtolin);
 	if (!res)
 		res = ast_register_translator(&lintoalaw);
