@@ -32,7 +32,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 206639 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 293969 $")
 
 #include <fcntl.h>
 #include <netinet/in.h>
@@ -56,8 +56,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 206639 $")
 #define G723_SAMPLES 240
 #define G729_SAMPLES 160
 
-static unsigned int global_useplc = 0;
-
 static struct channel_usage {
 	int total;
 	int encoders;
@@ -94,10 +92,10 @@ struct codec_dahdi_pvt {
 };
 
 /* Only used by a decoder */
-static int ulawtolin(struct ast_trans_pvt *pvt)
+static int ulawtolin(struct ast_trans_pvt *pvt, int samples)
 {
 	struct codec_dahdi_pvt *dahdip = pvt->pvt;
-	int i = dahdip->required_samples;
+	int i = samples;
 	uint8_t *src = &dahdip->ulaw_buffer[0];
 	int16_t *dst = pvt->outbuf.i16 + pvt->datalen;
 
@@ -336,7 +334,7 @@ static struct ast_frame *dahdi_decoder_frameout(struct ast_trans_pvt *pvt)
 		}
 	} else {
 		if (dahdip->softslin) {
-			ulawtolin(pvt);
+			ulawtolin(pvt, res);
 			pvt->f.datalen = res * 2;
 		} else {
 			pvt->f.datalen = res;
@@ -348,7 +346,7 @@ static struct ast_frame *dahdi_decoder_frameout(struct ast_trans_pvt *pvt)
 		pvt->f.offset = AST_FRIENDLY_OFFSET;
 		pvt->f.src = pvt->t->name;
 		pvt->f.data.ptr = pvt->outbuf.c;
-		pvt->f.samples = dahdip->required_samples;
+		pvt->f.samples = res;
 		ast_set_flag(&pvt->f, AST_FRFLAG_FROM_TRANSLATOR);
 		pvt->samples = 0;
 
@@ -493,29 +491,14 @@ static int register_translator(int dst, int src)
 	if (is_encoder(zt)) {
 		zt->t.framein = dahdi_encoder_framein;
 		zt->t.frameout = dahdi_encoder_frameout;
-#if 0
-		zt->t.buffer_samples = 0;
-#endif
 	} else {
 		zt->t.framein = dahdi_decoder_framein;
 		zt->t.frameout = dahdi_decoder_frameout;
-#if 0
-		if (AST_FORMAT_G723_1 == zt->t.srcfmt) {
-			zt->t.plc_samples = G723_SAMPLES;
-		} else {
-			zt->t.plc_samples = G729_SAMPLES;
-		}
-		zt->t.buffer_samples = zt->t.plc_samples * 8;
-#endif
 	}
 	zt->t.destroy = dahdi_destroy;
 	zt->t.buffer_samples = 0;
 	zt->t.newpvt = dahdi_new;
 	zt->t.sample = fakesrc_sample;
-#if 0
-	zt->t.useplc = global_useplc;
-#endif
-	zt->t.useplc = 0;
 	zt->t.native_plc = 0;
 
 	zt->t.desc_size = sizeof(struct codec_dahdi_pvt);
@@ -567,27 +550,6 @@ static void unregister_translators(void)
 	AST_LIST_UNLOCK(&translators);
 }
 
-static int parse_config(int reload)
-{
-	struct ast_variable *var;
-	struct ast_flags config_flags = { reload ? CONFIG_FLAG_FILEUNCHANGED : 0 };
-	struct ast_config *cfg = ast_config_load("codecs.conf", config_flags);
-
-	if (cfg == CONFIG_STATUS_FILEMISSING || cfg == CONFIG_STATUS_FILEUNCHANGED || cfg == CONFIG_STATUS_FILEINVALID)
-		return 0;
-
-	for (var = ast_variable_browse(cfg, "plc"); var; var = var->next) {
-	       if (!strcasecmp(var->name, "genericplc")) {
-		       global_useplc = ast_true(var->value);
-		       ast_verb(3, "codec_dahdi: %susing generic PLC\n",
-				global_useplc ? "" : "not ");
-	       }
-	}
-
-	ast_config_destroy(cfg);
-	return 0;
-}
-
 static void build_translators(struct format_map *map, unsigned int dstfmts, unsigned int srcfmts)
 {
 	unsigned int src, dst;
@@ -662,16 +624,6 @@ static int find_transcoders(void)
 
 static int reload(void)
 {
-	struct translator *cur;
-
-	if (parse_config(1))
-		return AST_MODULE_LOAD_DECLINE;
-
-	AST_LIST_LOCK(&translators);
-	AST_LIST_TRAVERSE(&translators, cur, entry)
-		cur->t.useplc = global_useplc;
-	AST_LIST_UNLOCK(&translators);
-
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
@@ -686,8 +638,6 @@ static int unload_module(void)
 static int load_module(void)
 {
 	ast_ulaw_init();
-	if (parse_config(0))
-		return AST_MODULE_LOAD_DECLINE;
 	find_transcoders();
 	ast_cli_register_multiple(cli, ARRAY_LEN(cli));
 	return AST_MODULE_LOAD_SUCCESS;
