@@ -29,7 +29,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 229285 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 267507 $")
 
 #include "asterisk/lock.h"
 #include "asterisk/linkedlists.h"
@@ -779,7 +779,6 @@ static struct ast_translator g726tolin = {
 	.desc_size = sizeof(struct g726_coder_pvt),
 	.buffer_samples = BUFFER_SAMPLES,
 	.buf_size = BUFFER_SAMPLES * 2,
-	.plc_samples = 160,
 };
 
 static struct ast_translator lintog726 = {
@@ -804,7 +803,6 @@ static struct ast_translator g726aal2tolin = {
 	.desc_size = sizeof(struct g726_coder_pvt),
 	.buffer_samples = BUFFER_SAMPLES,
 	.buf_size = BUFFER_SAMPLES * 2,
-	.plc_samples = 160,
 };
 
 static struct ast_translator lintog726aal2 = {
@@ -819,29 +817,8 @@ static struct ast_translator lintog726aal2 = {
 	.buf_size = BUFFER_SAMPLES / 2,
 };
 
-static int parse_config(int reload)
-{
-	struct ast_variable *var;
-	struct ast_flags config_flags = { reload ? CONFIG_FLAG_FILEUNCHANGED : 0 };
-	struct ast_config *cfg = ast_config_load("codecs.conf", config_flags);
-
-	if (cfg == CONFIG_STATUS_FILEMISSING || cfg == CONFIG_STATUS_FILEUNCHANGED || cfg == CONFIG_STATUS_FILEINVALID)
-		return 0;
-	for (var = ast_variable_browse(cfg, "plc"); var; var = var->next) {
-		if (!strcasecmp(var->name, "genericplc")) {
-			g726tolin.useplc = ast_true(var->value) ? 1 : 0;
-			ast_verb(3, "codec_g726: %susing generic PLC\n",
-					g726tolin.useplc ? "" : "not ");
-		}
-	}
-	ast_config_destroy(cfg);
-	return 0;
-}
-
 static int reload(void)
 {
-	if (parse_config(1))
-		return AST_MODULE_LOAD_DECLINE;
 	return AST_MODULE_LOAD_SUCCESS;
 }
 
@@ -862,10 +839,6 @@ static int load_module(void)
 {
 	int res = 0;
 
-
-	if (parse_config(0))
-		return AST_MODULE_LOAD_DECLINE;
-
 	res |= ast_register_translator(&g726tolin);
 	res |= ast_register_translator(&lintog726);
 
