@@ -24,7 +24,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 228649 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 301848 $")
 
 #include "asterisk/module.h"
 #include "asterisk/pbx.h"	/* function register/unregister */
@@ -43,6 +43,11 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 228649 $")
 		<description>
 			<para>Returns the base64 string.</para>
 		</description>
+		<see-also>
+			<ref type="function">BASE64_DECODE</ref>
+			<ref type="function">AES_DECRYPT</ref>
+			<ref type="function">AES_ENCRYPT</ref>
+		</see-also>
 	</function>
 	<function name="BASE64_DECODE" language="en_US">
 		<synopsis>
@@ -56,6 +61,11 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 228649 $")
 		<description>
 			<para>Returns the plain text string.</para>
 		</description>
+		<see-also>
+			<ref type="function">BASE64_ENCODE</ref>
+			<ref type="function">AES_DECRYPT</ref>
+			<ref type="function">AES_ENCRYPT</ref>
+		</see-also>
 	</function>
  ***/
 
