@@ -23,7 +23,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 220632 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 280812 $")
 
 #include "asterisk/module.h"
 #include "asterisk/channel.h"
@@ -191,6 +191,7 @@ static int callerid_read(struct ast_channel *chan, const char *cmd, char *data,
 static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 			  const char *value)
 {
+	int valid = 1;
 	if (!value || !chan)
 		return -1;
 
@@ -267,6 +268,11 @@ static int callerid_write(struct ast_channel *chan, const char *cmd, char *data,
 		chan->cid.cid_ton = atoi(value);
 	} else {
 		ast_log(LOG_ERROR, "Unknown callerid data type '%s'.\n", data);
+		valid = 0;
+	}
+
+	if (valid) {
+		chan->cid.cid_tns = 1;
 	}
 
 	return 0;
