@@ -26,7 +26,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 250733 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 314205 $")
 
 #include <regex.h>
 
@@ -165,14 +165,6 @@ ASTERISK_FILE_VERSION(__FILE__, "$Revision: 250733 $")
 						<para>     <literal>remote_normdevjitter</literal>Remote calculated jitter (normal deviation)</para>
 						<para>     <literal>remote_stdevjitter</literal>Remote calculated jitter (standard deviation)</para>
 						<para>     <literal>remote_count</literal>      Number of transmitted packets</para>
-						<para>     <literal>remote_ssrc</literal>       Remote SSRC (stream ID)</para>
-						<para>     <literal>remote_lostpackets</literal>Remote lost packets</para>
-						<para>     <literal>remote_jitter</literal>     Remote reported jitter</para>
-						<para>     <literal>remote_maxjitter</literal>  Remote calculated jitter (maximum)</para>
-						<para>     <literal>remote_minjitter</literal>  Remote calculated jitter (minimum)</para>
-						<para>     <literal>remote_normdevjitter</literal>Remote calculated jitter (normal deviation)</para>
-						<para>     <literal>remote_stdevjitter</literal>Remote calculated jitter (standard deviation)</para>
-						<para>     <literal>remote_count</literal>      Number of transmitted packets</para>
 						<para>     <literal>rtt</literal>               Round trip time</para>
 						<para>     <literal>maxrtt</literal>            Round trip time (maximum)</para>
 						<para>     <literal>minrtt</literal>            Round trip time (minimum)</para>
@@ -278,7 +270,7 @@ static int func_channel_read(struct ast_channel *chan, const char *function,
 	return ret;
 }
 
-static int func_channel_write(struct ast_channel *chan, const char *function,
+static int func_channel_write_real(struct ast_channel *chan, const char *function,
 			      char *data, const char *value)
 {
 	int ret = 0;
@@ -344,6 +336,24 @@ static int func_channel_write(struct ast_channel *chan, const char *function,
 	return ret;
 }
 
+static int func_channel_write(struct ast_channel *chan, const char *function, char *data, const char *value)
+{
+	int res;
+	ast_chan_write_info_t write_info = {
+		.version = AST_CHAN_WRITE_INFO_T_VERSION,
+		.write_fn = func_channel_write_real,
+		.chan = chan,
+		.function = function,
+		.data = data,
+		.value = value,
+	};
+
+	res = func_channel_write_real(chan, function, data, value);
+	ast_channel_setoption(chan, AST_OPTION_CHANNEL_WRITE, &write_info, sizeof(write_info), 0);
+
+	return res;
+}
+
 static struct ast_custom_function channel_function = {
 	.name = "CHANNEL",
 	.read = func_channel_read,
