@@ -33,7 +33,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 261947 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 310414 $")
 
 #include "asterisk/module.h"
 #include "asterisk/file.h"
@@ -170,7 +170,7 @@ static SQLHSTMT generic_execute(struct odbc_obj *obj, void *data)
 	}
 
 	res = SQLExecDirect(stmt, (unsigned char *)sql, SQL_NTS);
-	if ((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO)) {
+	if ((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO) && (res != SQL_NO_DATA)) {
 		if (res == SQL_ERROR) {
 			int i;
 			SQLINTEGER nativeerror=0, numfields=0;
@@ -219,7 +219,7 @@ static int acf_odbc_write(struct ast_channel *chan, const char *cmd, char *s, co
 	struct ast_str *insertbuf = ast_str_thread_get(&sql2_buf, 16);
 	const char *status = "FAILURE";
 
-	if (!buf) {
+	if (!buf || !insertbuf) {
 		return -1;
 	}
 
@@ -302,12 +302,12 @@ static int acf_odbc_write(struct ast_channel *chan, const char *cmd, char *s, co
 	 * transaction, or else we CANNOT enforce atomicity.
 	 */
 	for (dsn = 0; dsn < 5; dsn++) {
-		if (transactional) {
-			/* This can only happen second time through or greater. */
-			ast_log(LOG_WARNING, "Transactions do not work well with multiple DSNs for 'writehandle'\n");
-		}
-
 		if (!ast_strlen_zero(query->writehandle[dsn])) {
+			if (transactional) {
+				/* This can only happen second time through or greater. */
+				ast_log(LOG_WARNING, "Transactions do not work well with multiple DSNs for 'writehandle'\n");
+			}
+
 			if ((obj = ast_odbc_retrieve_transaction_obj(chan, query->writehandle[dsn]))) {
 				transactional = 1;
 			} else {
@@ -325,12 +325,34 @@ static int acf_odbc_write(struct ast_channel *chan, const char *cmd, char *s, co
 		}
 	}
 
+	if (stmt) {
+		SQLRowCount(stmt, &rows);
+	}
+
 	if (stmt && rows == 0 && ast_str_strlen(insertbuf) != 0) {
 		SQLCloseCursor(stmt);
 		SQLFreeHandle(SQL_HANDLE_STMT, stmt);
-		for (dsn = 0; dsn < 5; dsn++) {
+		if (obj && !transactional) {
+			ast_odbc_release_obj(obj);
+			obj = NULL;
+		}
+
+		for (transactional = 0, dsn = 0; dsn < 5; dsn++) {
 			if (!ast_strlen_zero(query->writehandle[dsn])) {
-				obj = ast_odbc_request_obj(query->writehandle[dsn], 0);
+				if (transactional) {
+					/* This can only happen second time through or greater. */
+					ast_log(LOG_WARNING, "Transactions do not work well with multiple DSNs for 'writehandle'\n");
+				} else if (obj) {
+					ast_odbc_release_obj(obj);
+					obj = NULL;
+				}
+
+				if ((obj = ast_odbc_retrieve_transaction_obj(chan, query->writehandle[dsn]))) {
+					transactional = 1;
+				} else {
+					obj = ast_odbc_request_obj(query->writehandle[dsn], 0);
+					transactional = 0;
+				}
 				if (obj) {
 					stmt = ast_odbc_direct_execute(obj, generic_execute, ast_str_buffer(insertbuf));
 				}
@@ -340,14 +362,9 @@ static int acf_odbc_write(struct ast_channel *chan, const char *cmd, char *s, co
 				SQLRowCount(stmt, &rows);
 				break;
 			}
-			if (obj) {
-				ast_odbc_release_obj(obj);
-				obj = NULL;
-			}
 		}
 	} else if (stmt) {
 		status = "SUCCESS";
-		SQLRowCount(stmt, &rows);
 	}
 
 	AST_RWLIST_UNLOCK(&queries);
@@ -369,10 +386,12 @@ static int acf_odbc_write(struct ast_channel *chan, const char *cmd, char *s, co
 		obj = NULL;
 	}
 
-	if (chan)
+	if (chan) {
 		ast_autoservice_stop(chan);
-	if (bogus_chan)
+	}
+	if (bogus_chan) {
 		ast_channel_free(chan);
+	}
 
 	return 0;
 }
@@ -396,7 +415,7 @@ static int acf_odbc_read(struct ast_channel *chan, const char *cmd, char *s, cha
 	struct ast_str *sql = ast_str_thread_get(&sql_buf, 16);
 	const char *status = "FAILURE";
 
-	if (!sql) {
+	if (!sql || !colnames) {
 		pbx_builtin_setvar_helper(chan, "ODBCSTATUS", status);
 		return -1;
 	}
@@ -539,9 +558,24 @@ static int acf_odbc_read(struct ast_channel *chan, const char *cmd, char *s, cha
 			struct ast_str *coldata = ast_str_thread_get(&coldata_buf, 16);
 			char *ptrcoldata;
 
+			if (!coldata) {
+				ast_free(resultset);
+				SQLCloseCursor(stmt);
+				SQLFreeHandle(SQL_HANDLE_STMT, stmt);
+				ast_odbc_release_obj(obj);
+				obj = NULL;
+				pbx_builtin_setvar_helper(chan, "ODBCSTATUS", "MEMERROR");
+				if (chan)
+					ast_autoservice_stop(chan);
+				if (bogus_chan) {
+					ast_channel_free(chan);
+				}
+				return -1;
+			}
+
 			if (y == 0) {
 				char colname[256];
-				SQLULEN maxcol;
+				SQLULEN maxcol = 0;
 
 				res = SQLDescribeCol(stmt, x + 1, (unsigned char *)colname, sizeof(colname), &collength, NULL, &maxcol, NULL, NULL);
 				ast_debug(3, "Got collength of %d and maxcol of %d for column '%s' (offset %d)\n", (int)collength, (int)maxcol, colname, x);
@@ -1083,6 +1117,11 @@ static char *cli_odbc_read(struct ast_cli_entry *e, int cmd, struct ast_cli_args
 		char colname[256];
 		SQLULEN maxcol;
 
+		if (!coldata) {
+			AST_RWLIST_UNLOCK(&queries);
+			return CLI_SUCCESS;
+		}
+
 		for (dsn = 0; dsn < 5; dsn++) {
 			if (ast_strlen_zero(query->readhandle[dsn])) {
 				continue;
@@ -1129,6 +1168,8 @@ static char *cli_odbc_read(struct ast_cli_entry *e, int cmd, struct ast_cli_args
 			}
 			for (;;) {
 				for (x = 0; x < colcount; x++) {
+					maxcol = 0;
+
 					res = SQLDescribeCol(stmt, x + 1, (unsigned char *)colname, sizeof(colname), &collength, NULL, &maxcol, NULL, NULL);
 					if (((res != SQL_SUCCESS) && (res != SQL_SUCCESS_WITH_INFO)) || collength == 0) {
 						snprintf(colname, sizeof(colname), "field%d", x);
