@@ -31,7 +31,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 251632 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 297311 $")
 
 #include "asterisk/frame.h"
 #include "asterisk/channel.h"
@@ -763,6 +763,11 @@ static void jb_destroy_adaptive(void *jb)
 
 static int jb_put_first_adaptive(void *jb, struct ast_frame *fin, long now)
 {
+	jitterbuf *adaptivejb = (jitterbuf *) jb;
+
+	/* Initialize the offset to that of the first frame's timestamp */
+	adaptivejb->info.resync_offset = fin->ts;
+
 	return jb_put_adaptive(jb, fin, now);
 }
 
