@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -47,7 +46,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3"
+#define YYBISON_VERSION "2.4.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -55,79 +54,28 @@
 /* Pure parsers.  */
 #define YYPURE 1
 
+/* Push parsers.  */
+#define YYPUSH 0
+
+/* Pull parsers.  */
+#define YYPULL 1
+
 /* Using locations.  */
 #define YYLSP_NEEDED 1
 
 /* Substitute the variable and function names.  */
-#define yyparse ast_yyparse
-#define yylex   ast_yylex
-#define yyerror ast_yyerror
-#define yylval  ast_yylval
-#define yychar  ast_yychar
-#define yydebug ast_yydebug
-#define yynerrs ast_yynerrs
-#define yylloc ast_yylloc
-
-/* Tokens.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     TOK_COMMA = 258,
-     TOK_COLONCOLON = 259,
-     TOK_COND = 260,
-     TOK_OR = 261,
-     TOK_AND = 262,
-     TOK_NE = 263,
-     TOK_LE = 264,
-     TOK_GE = 265,
-     TOK_LT = 266,
-     TOK_GT = 267,
-     TOK_EQ = 268,
-     TOK_MINUS = 269,
-     TOK_PLUS = 270,
-     TOK_MOD = 271,
-     TOK_DIV = 272,
-     TOK_MULT = 273,
-     TOK_COMPL = 274,
-     TOK_TILDETILDE = 275,
-     TOK_EQTILDE = 276,
-     TOK_COLON = 277,
-     TOK_LP = 278,
-     TOK_RP = 279,
-     TOKEN = 280
-   };
-#endif
-/* Tokens.  */
-#define TOK_COMMA 258
-#define TOK_COLONCOLON 259
-#define TOK_COND 260
-#define TOK_OR 261
-#define TOK_AND 262
-#define TOK_NE 263
-#define TOK_LE 264
-#define TOK_GE 265
-#define TOK_LT 266
-#define TOK_GT 267
-#define TOK_EQ 268
-#define TOK_MINUS 269
-#define TOK_PLUS 270
-#define TOK_MOD 271
-#define TOK_DIV 272
-#define TOK_MULT 273
-#define TOK_COMPL 274
-#define TOK_TILDETILDE 275
-#define TOK_EQTILDE 276
-#define TOK_COLON 277
-#define TOK_LP 278
-#define TOK_RP 279
-#define TOKEN 280
-
-
-
+#define yyparse         ast_yyparse
+#define yylex           ast_yylex
+#define yyerror         ast_yyerror
+#define yylval          ast_yylval
+#define yychar          ast_yychar
+#define yydebug         ast_yydebug
+#define yynerrs         ast_yynerrs
+#define yylloc          ast_yylloc
 
 /* Copy the first part of user declarations.  */
+
+/* Line 189 of yacc.c  */
 #line 1 "ast_expr2.y"
 
 /* Written by Pace Willisson (pace@blitz.com) 
@@ -150,7 +98,7 @@
 
 #if !defined(STANDALONE) && !defined(STANDALONE2)	\
 	
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 236716 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 293196 $")
 #else
 #ifndef __USE_ISOC99
 #define __USE_ISOC99 1
@@ -466,6 +414,9 @@ int		ast_yyerror(const char *,YYLTYPE *, struct parse_io *);
 #define DESTROY(x) {if((x)->type == AST_EXPR_numeric_string || (x)->type == AST_EXPR_string) free((x)->u.s); (x)->u.s = 0; free(x);}
 
 
+/* Line 189 of yacc.c  */
+#line 419 "ast_expr2.c"
+
 /* Enabling traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -484,19 +435,59 @@ int		ast_yyerror(const char *,YYLTYPE *, struct parse_io *);
 # define YYTOKEN_TABLE 0
 #endif
 
+
+/* Tokens.  */
+#ifndef YYTOKENTYPE
+# define YYTOKENTYPE
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum yytokentype {
+     TOK_COMMA = 258,
+     TOK_COLONCOLON = 259,
+     TOK_COND = 260,
+     TOK_OR = 261,
+     TOK_AND = 262,
+     TOK_NE = 263,
+     TOK_LE = 264,
+     TOK_GE = 265,
+     TOK_LT = 266,
+     TOK_GT = 267,
+     TOK_EQ = 268,
+     TOK_MINUS = 269,
+     TOK_PLUS = 270,
+     TOK_MOD = 271,
+     TOK_DIV = 272,
+     TOK_MULT = 273,
+     TOK_COMPL = 274,
+     TOK_TILDETILDE = 275,
+     TOK_EQTILDE = 276,
+     TOK_COLON = 277,
+     TOK_LP = 278,
+     TOK_RP = 279,
+     TOKEN = 280
+   };
+#endif
+
+
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 345 "ast_expr2.y"
 {
+
+/* Line 214 of yacc.c  */
+#line 345 "ast_expr2.y"
+
 	struct val *val;
 	struct expr_node *arglist;
-}
-/* Line 187 of yacc.c.  */
-#line 496 "ast_expr2.c"
-	YYSTYPE;
+
+
+
+/* Line 214 of yacc.c  */
+#line 487 "ast_expr2.c"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 #if ! defined YYLTYPE && ! defined YYLTYPE_IS_DECLARED
@@ -514,13 +505,15 @@ typedef struct YYLTYPE
 
 
 /* Copy the second part of user declarations.  */
+
+/* Line 264 of yacc.c  */
 #line 350 "ast_expr2.y"
 
 extern int		ast_yylex __P((YYSTYPE *, YYLTYPE *, yyscan_t));
 
 
-/* Line 216 of yacc.c.  */
-#line 524 "ast_expr2.c"
+/* Line 264 of yacc.c  */
+#line 517 "ast_expr2.c"
 
 #ifdef short
 # undef short
@@ -595,14 +588,14 @@ typedef short int yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int i)
+YYID (int yyi)
 #else
 static int
-YYID (i)
-    int i;
+YYID (yyi)
+    int yyi;
 #endif
 {
-  return i;
+  return yyi;
 }
 #endif
 
@@ -684,9 +677,9 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  yytype_int16 yyss;
-  YYSTYPE yyvs;
-    YYLTYPE yyls;
+  yytype_int16 yyss_alloc;
+  YYSTYPE yyvs_alloc;
+  YYLTYPE yyls_alloc;
 };
 
 /* The size of the maximum gap between one aligned stack and the next.  */
@@ -721,12 +714,12 @@ union yyalloc
    elements in the stack, and YYPTR gives the new location of the
    stack.  Advance YYPTR to a properly aligned location for the next
    stack.  */
-# define YYSTACK_RELOCATE(Stack)					\
+# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
     do									\
       {									\
 	YYSIZE_T yynewbytes;						\
-	YYCOPY (&yyptr->Stack, Stack, yysize);				\
-	Stack = &yyptr->Stack;						\
+	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
+	Stack = &yyptr->Stack_alloc;					\
 	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
@@ -1146,17 +1139,20 @@ yy_symbol_print (yyoutput, yytype, yyvaluep, yylocationp)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
+yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
 #else
 static void
-yy_stack_print (bottom, top)
-    yytype_int16 *bottom;
-    yytype_int16 *top;
+yy_stack_print (yybottom, yytop)
+    yytype_int16 *yybottom;
+    yytype_int16 *yytop;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; bottom <= top; ++bottom)
-    YYFPRINTF (stderr, " %d", *bottom);
+  for (; yybottom <= yytop; yybottom++)
+    {
+      int yybot = *yybottom;
+      YYFPRINTF (stderr, " %d", yybot);
+    }
   YYFPRINTF (stderr, "\n");
 }
 
@@ -1191,11 +1187,11 @@ yy_reduce_print (yyvsp, yylsp, yyrule)
   /* The symbols being reduced.  */
   for (yyi = 0; yyi < yynrhs; yyi++)
     {
-      fprintf (stderr, "   $%d = ", yyi + 1);
+      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
 		       &(yyvsp[(yyi + 1) - (yynrhs)])
 		       , &(yylsp[(yyi + 1) - (yynrhs)])		       );
-      fprintf (stderr, "\n");
+      YYFPRINTF (stderr, "\n");
     }
 }
 
@@ -1473,129 +1469,219 @@ yydestruct (yymsg, yytype, yyvaluep, yylocationp)
   switch (yytype)
     {
       case 4: /* "TOK_COLONCOLON" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
 #line 1479 "ast_expr2.c"
 	break;
       case 5: /* "TOK_COND" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1484 "ast_expr2.c"
+
+/* Line 1000 of yacc.c  */
+#line 1488 "ast_expr2.c"
 	break;
       case 6: /* "TOK_OR" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1489 "ast_expr2.c"
+
+/* Line 1000 of yacc.c  */
+#line 1497 "ast_expr2.c"
 	break;
       case 7: /* "TOK_AND" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1494 "ast_expr2.c"
+
+/* Line 1000 of yacc.c  */
+#line 1506 "ast_expr2.c"
 	break;
       case 8: /* "TOK_NE" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1499 "ast_expr2.c"
+
+/* Line 1000 of yacc.c  */
+#line 1515 "ast_expr2.c"
 	break;
       case 9: /* "TOK_LE" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1504 "ast_expr2.c"
-	break;
-      case 10: /* "TOK_GE" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1509 "ast_expr2.c"
-	break;
-      case 11: /* "TOK_LT" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1514 "ast_expr2.c"
-	break;
-      case 12: /* "TOK_GT" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1519 "ast_expr2.c"
-	break;
-      case 13: /* "TOK_EQ" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
 #line 1524 "ast_expr2.c"
 	break;
+      case 10: /* "TOK_GE" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1533 "ast_expr2.c"
+	break;
+      case 11: /* "TOK_LT" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1542 "ast_expr2.c"
+	break;
+      case 12: /* "TOK_GT" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1551 "ast_expr2.c"
+	break;
+      case 13: /* "TOK_EQ" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1560 "ast_expr2.c"
+	break;
       case 14: /* "TOK_MINUS" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1529 "ast_expr2.c"
-	break;
-      case 15: /* "TOK_PLUS" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1534 "ast_expr2.c"
-	break;
-      case 16: /* "TOK_MOD" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1539 "ast_expr2.c"
-	break;
-      case 17: /* "TOK_DIV" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1544 "ast_expr2.c"
-	break;
-      case 18: /* "TOK_MULT" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1549 "ast_expr2.c"
-	break;
-      case 19: /* "TOK_COMPL" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1554 "ast_expr2.c"
-	break;
-      case 20: /* "TOK_TILDETILDE" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1559 "ast_expr2.c"
-	break;
-      case 21: /* "TOK_EQTILDE" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
-#line 1564 "ast_expr2.c"
-	break;
-      case 22: /* "TOK_COLON" */
-#line 368 "ast_expr2.y"
-	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
 #line 1569 "ast_expr2.c"
 	break;
-      case 23: /* "TOK_LP" */
+      case 15: /* "TOK_PLUS" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1574 "ast_expr2.c"
+
+/* Line 1000 of yacc.c  */
+#line 1578 "ast_expr2.c"
+	break;
+      case 16: /* "TOK_MOD" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1587 "ast_expr2.c"
+	break;
+      case 17: /* "TOK_DIV" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1596 "ast_expr2.c"
+	break;
+      case 18: /* "TOK_MULT" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1605 "ast_expr2.c"
+	break;
+      case 19: /* "TOK_COMPL" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1614 "ast_expr2.c"
+	break;
+      case 20: /* "TOK_TILDETILDE" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1623 "ast_expr2.c"
+	break;
+      case 21: /* "TOK_EQTILDE" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1632 "ast_expr2.c"
+	break;
+      case 22: /* "TOK_COLON" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1641 "ast_expr2.c"
+	break;
+      case 23: /* "TOK_LP" */
+
+/* Line 1000 of yacc.c  */
+#line 368 "ast_expr2.y"
+	{  free_value((yyvaluep->val)); };
+
+/* Line 1000 of yacc.c  */
+#line 1650 "ast_expr2.c"
 	break;
       case 24: /* "TOK_RP" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1579 "ast_expr2.c"
+
+/* Line 1000 of yacc.c  */
+#line 1659 "ast_expr2.c"
 	break;
       case 25: /* "TOKEN" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1584 "ast_expr2.c"
+
+/* Line 1000 of yacc.c  */
+#line 1668 "ast_expr2.c"
 	break;
       case 29: /* "expr" */
+
+/* Line 1000 of yacc.c  */
 #line 368 "ast_expr2.y"
 	{  free_value((yyvaluep->val)); };
-#line 1589 "ast_expr2.c"
+
+/* Line 1000 of yacc.c  */
+#line 1677 "ast_expr2.c"
 	break;
 
       default:
 	break;
     }
 }
-
 
 /* Prevent warnings from -Wmissing-prototypes.  */
-
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
 int yyparse (void *YYPARSE_PARAM);
@@ -1614,10 +1700,9 @@ int yyparse ();
 
 
 
-
-/*----------.
-| yyparse.  |
-`----------*/
+/*-------------------------.
+| yyparse or yypush_parse.  |
+`-------------------------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1641,24 +1726,59 @@ yyparse ()
 #endif
 #endif
 {
-  /* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
-/* Number of syntax errors so far.  */
-int yynerrs;
-/* Location data for the look-ahead symbol.  */
+/* Location data for the lookahead symbol.  */
 YYLTYPE yylloc;
 
-  int yystate;
+    /* Number of syntax errors so far.  */
+    int yynerrs;
+
+    int yystate;
+    /* Number of tokens to shift before error messages enabled.  */
+    int yyerrstatus;
+
+    /* The stacks and their tools:
+       `yyss': related to states.
+       `yyvs': related to semantic values.
+       `yyls': related to locations.
+
+       Refer to the stacks thru separate pointers, to allow yyoverflow
+       to reallocate them elsewhere.  */
+
+    /* The state stack.  */
+    yytype_int16 yyssa[YYINITDEPTH];
+    yytype_int16 *yyss;
+    yytype_int16 *yyssp;
+
+    /* The semantic value stack.  */
+    YYSTYPE yyvsa[YYINITDEPTH];
+    YYSTYPE *yyvs;
+    YYSTYPE *yyvsp;
+
+    /* The location stack.  */
+    YYLTYPE yylsa[YYINITDEPTH];
+    YYLTYPE *yyls;
+    YYLTYPE *yylsp;
+
+    /* The locations where the error started and ended.  */
+    YYLTYPE yyerror_range[2];
+
+    YYSIZE_T yystacksize;
+
   int yyn;
   int yyresult;
-  /* Number of tokens to shift before error messages enabled.  */
-  int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  /* Lookahead token as an internal (translated) token number.  */
+  int yytoken;
+  /* The variables used to return semantic value and location from the
+     action routines.  */
+  YYSTYPE yyval;
+  YYLTYPE yyloc;
+
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
   char yymsgbuf[128];
@@ -1666,63 +1786,37 @@ YYLTYPE yylloc;
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-  /* Three stacks and their tools:
-     `yyss': related to states,
-     `yyvs': related to semantic values,
-     `yyls': related to locations.
-
-     Refer to the stacks thru separate pointers, to allow yyoverflow
-     to reallocate them elsewhere.  */
-
-  /* The state stack.  */
-  yytype_int16 yyssa[YYINITDEPTH];
-  yytype_int16 *yyss = yyssa;
-  yytype_int16 *yyssp;
-
-  /* The semantic value stack.  */
-  YYSTYPE yyvsa[YYINITDEPTH];
-  YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
-
-  /* The location stack.  */
-  YYLTYPE yylsa[YYINITDEPTH];
-  YYLTYPE *yyls = yylsa;
-  YYLTYPE *yylsp;
-  /* The locations where the error started and ended.  */
-  YYLTYPE yyerror_range[2];
-
 #define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N), yylsp -= (N))
 
-  YYSIZE_T yystacksize = YYINITDEPTH;
-
-  /* The variables used to return semantic value and location from the
-     action routines.  */
-  YYSTYPE yyval;
-  YYLTYPE yyloc;
-
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
+  yyls = yylsa;
+  yystacksize = YYINITDEPTH;
+
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yystate = 0;
   yyerrstatus = 0;
   yynerrs = 0;
-  yychar = YYEMPTY;		/* Cause a token to be read.  */
+  yychar = YYEMPTY; /* Cause a token to be read.  */
 
   /* Initialize stack pointers.
      Waste one element of value and location stack
      so that they stay on the same level as the state stack.
      The wasted elements are never initialized.  */
-
   yyssp = yyss;
   yyvsp = yyvs;
   yylsp = yyls;
+
 #if YYLTYPE_IS_TRIVIAL
   /* Initialize the default location before parsing starts.  */
   yylloc.first_line   = yylloc.last_line   = 1;
-  yylloc.first_column = yylloc.last_column = 0;
+  yylloc.first_column = yylloc.last_column = 1;
 #endif
 
   goto yysetstate;
@@ -1761,6 +1855,7 @@ YYLTYPE yylloc;
 		    &yyvs1, yysize * sizeof (*yyvsp),
 		    &yyls1, yysize * sizeof (*yylsp),
 		    &yystacksize);
+
 	yyls = yyls1;
 	yyss = yyss1;
 	yyvs = yyvs1;
@@ -1782,9 +1877,9 @@ YYLTYPE yylloc;
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
 	  goto yyexhaustedlab;
-	YYSTACK_RELOCATE (yyss);
-	YYSTACK_RELOCATE (yyvs);
-	YYSTACK_RELOCATE (yyls);
+	YYSTACK_RELOCATE (yyss_alloc, yyss);
+	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
+	YYSTACK_RELOCATE (yyls_alloc, yyls);
 #  undef YYSTACK_RELOCATE
 	if (yyss1 != yyssa)
 	  YYSTACK_FREE (yyss1);
@@ -1805,6 +1900,9 @@ YYLTYPE yylloc;
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
+  if (yystate == YYFINAL)
+    YYACCEPT;
+
   goto yybackup;
 
 /*-----------.
@@ -1813,16 +1911,16 @@ YYLTYPE yylloc;
 yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     look-ahead token if we need one and don't already have one.  */
+     lookahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1854,20 +1952,16 @@ yybackup:
       goto yyreduce;
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (yyerrstatus)
     yyerrstatus--;
 
-  /* Shift the look-ahead token.  */
+  /* Shift the lookahead token.  */
   YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
 
-  /* Discard the shifted token unless it is eof.  */
-  if (yychar != YYEOF)
-    yychar = YYEMPTY;
+  /* Discard the shifted token.  */
+  yychar = YYEMPTY;
 
   yystate = yyn;
   *++yyvsp = yylval;
@@ -1908,6 +2002,8 @@ yyreduce:
   switch (yyn)
     {
         case 2:
+
+/* Line 1455 of yacc.c  */
 #line 374 "ast_expr2.y"
     { ((struct parse_io *)parseio)->val = (struct val *)calloc(sizeof(struct val),1);
               ((struct parse_io *)parseio)->val->type = (yyvsp[(1) - (1)].val)->type;
@@ -1920,6 +2016,8 @@ yyreduce:
     break;
 
   case 3:
+
+/* Line 1455 of yacc.c  */
 #line 382 "ast_expr2.y"
     {/* nothing */ ((struct parse_io *)parseio)->val = (struct val *)calloc(sizeof(struct val),1);
               ((struct parse_io *)parseio)->val->type = AST_EXPR_string;
@@ -1928,11 +2026,15 @@ yyreduce:
     break;
 
   case 4:
+
+/* Line 1455 of yacc.c  */
 #line 389 "ast_expr2.y"
     { (yyval.arglist) = alloc_expr_node(AST_EXPR_NODE_VAL); (yyval.arglist)->val = (yyvsp[(1) - (1)].val);;}
     break;
 
   case 5:
+
+/* Line 1455 of yacc.c  */
 #line 390 "ast_expr2.y"
     {struct expr_node *x = alloc_expr_node(AST_EXPR_NODE_VAL);
                                  struct expr_node *t;
@@ -1943,6 +2045,8 @@ yyreduce:
     break;
 
   case 6:
+
+/* Line 1455 of yacc.c  */
 #line 396 "ast_expr2.y"
     {struct expr_node *x = alloc_expr_node(AST_EXPR_NODE_VAL);
                                  struct expr_node *t;  /* NULL args should OK */
@@ -1953,6 +2057,8 @@ yyreduce:
     break;
 
   case 7:
+
+/* Line 1455 of yacc.c  */
 #line 405 "ast_expr2.y"
     { (yyval.val) = op_func((yyvsp[(1) - (4)].val),(yyvsp[(3) - (4)].arglist), ((struct parse_io *)parseio)->chan);
 		                            DESTROY((yyvsp[(2) - (4)].val));
@@ -1963,11 +2069,15 @@ yyreduce:
     break;
 
   case 8:
+
+/* Line 1455 of yacc.c  */
 #line 411 "ast_expr2.y"
     {(yyval.val) = (yyvsp[(1) - (1)].val);;}
     break;
 
   case 9:
+
+/* Line 1455 of yacc.c  */
 #line 412 "ast_expr2.y"
     { (yyval.val) = (yyvsp[(2) - (3)].val);
 	                       (yyloc).first_column = (yylsp[(1) - (3)]).first_column; (yyloc).last_column = (yylsp[(3) - (3)]).last_column; 
@@ -1976,6 +2086,8 @@ yyreduce:
     break;
 
   case 10:
+
+/* Line 1455 of yacc.c  */
 #line 416 "ast_expr2.y"
     { (yyval.val) = op_or ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val));
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -1984,6 +2096,8 @@ yyreduce:
     break;
 
   case 11:
+
+/* Line 1455 of yacc.c  */
 #line 420 "ast_expr2.y"
     { (yyval.val) = op_and ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -1992,6 +2106,8 @@ yyreduce:
     break;
 
   case 12:
+
+/* Line 1455 of yacc.c  */
 #line 424 "ast_expr2.y"
     { (yyval.val) = op_eq ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val));
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2000,6 +2116,8 @@ yyreduce:
     break;
 
   case 13:
+
+/* Line 1455 of yacc.c  */
 #line 428 "ast_expr2.y"
     { (yyval.val) = op_gt ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val));
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2008,6 +2126,8 @@ yyreduce:
     break;
 
   case 14:
+
+/* Line 1455 of yacc.c  */
 #line 432 "ast_expr2.y"
     { (yyval.val) = op_lt ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2016,6 +2136,8 @@ yyreduce:
     break;
 
   case 15:
+
+/* Line 1455 of yacc.c  */
 #line 436 "ast_expr2.y"
     { (yyval.val) = op_ge ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2024,6 +2146,8 @@ yyreduce:
     break;
 
   case 16:
+
+/* Line 1455 of yacc.c  */
 #line 440 "ast_expr2.y"
     { (yyval.val) = op_le ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2032,6 +2156,8 @@ yyreduce:
     break;
 
   case 17:
+
+/* Line 1455 of yacc.c  */
 #line 444 "ast_expr2.y"
     { (yyval.val) = op_ne ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2040,6 +2166,8 @@ yyreduce:
     break;
 
   case 18:
+
+/* Line 1455 of yacc.c  */
 #line 448 "ast_expr2.y"
     { (yyval.val) = op_plus ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2048,6 +2176,8 @@ yyreduce:
     break;
 
   case 19:
+
+/* Line 1455 of yacc.c  */
 #line 452 "ast_expr2.y"
     { (yyval.val) = op_minus ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2056,6 +2186,8 @@ yyreduce:
     break;
 
   case 20:
+
+/* Line 1455 of yacc.c  */
 #line 456 "ast_expr2.y"
     { (yyval.val) = op_negate ((yyvsp[(2) - (2)].val)); 
 						DESTROY((yyvsp[(1) - (2)].val));	
@@ -2064,6 +2196,8 @@ yyreduce:
     break;
 
   case 21:
+
+/* Line 1455 of yacc.c  */
 #line 460 "ast_expr2.y"
     { (yyval.val) = op_compl ((yyvsp[(2) - (2)].val)); 
 						DESTROY((yyvsp[(1) - (2)].val));	
@@ -2072,6 +2206,8 @@ yyreduce:
     break;
 
   case 22:
+
+/* Line 1455 of yacc.c  */
 #line 464 "ast_expr2.y"
     { (yyval.val) = op_times ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2080,6 +2216,8 @@ yyreduce:
     break;
 
   case 23:
+
+/* Line 1455 of yacc.c  */
 #line 468 "ast_expr2.y"
     { (yyval.val) = op_div ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2088,6 +2226,8 @@ yyreduce:
     break;
 
   case 24:
+
+/* Line 1455 of yacc.c  */
 #line 472 "ast_expr2.y"
     { (yyval.val) = op_rem ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2096,6 +2236,8 @@ yyreduce:
     break;
 
   case 25:
+
+/* Line 1455 of yacc.c  */
 #line 476 "ast_expr2.y"
     { (yyval.val) = op_colon ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2104,6 +2246,8 @@ yyreduce:
     break;
 
   case 26:
+
+/* Line 1455 of yacc.c  */
 #line 480 "ast_expr2.y"
     { (yyval.val) = op_eqtilde ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2112,6 +2256,8 @@ yyreduce:
     break;
 
   case 27:
+
+/* Line 1455 of yacc.c  */
 #line 484 "ast_expr2.y"
     { (yyval.val) = op_cond ((yyvsp[(1) - (5)].val), (yyvsp[(3) - (5)].val), (yyvsp[(5) - (5)].val)); 
 						DESTROY((yyvsp[(2) - (5)].val));	
@@ -2121,6 +2267,8 @@ yyreduce:
     break;
 
   case 28:
+
+/* Line 1455 of yacc.c  */
 #line 489 "ast_expr2.y"
     { (yyval.val) = op_tildetilde ((yyvsp[(1) - (3)].val), (yyvsp[(3) - (3)].val)); 
 						DESTROY((yyvsp[(2) - (3)].val));	
@@ -2129,8 +2277,9 @@ yyreduce:
     break;
 
 
-/* Line 1267 of yacc.c.  */
-#line 2134 "ast_expr2.c"
+
+/* Line 1455 of yacc.c  */
+#line 2283 "ast_expr2.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -2206,7 +2355,7 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
@@ -2223,7 +2372,7 @@ yyerrlab:
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -2281,14 +2430,11 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   *++yyvsp = yylval;
 
   yyerror_range[1] = yylloc;
   /* Using YYLLOC is tempting, but would change the location of
-     the look-ahead.  YYLOC is available though.  */
+     the lookahead.  YYLOC is available though.  */
   YYLLOC_DEFAULT (yyloc, (yyerror_range - 1), 2);
   *++yylsp = yyloc;
 
@@ -2313,7 +2459,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#ifndef yyoverflow
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -2324,7 +2470,7 @@ yyexhaustedlab:
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
+  if (yychar != YYEMPTY)
      yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval, &yylloc);
   /* Do not reclaim the symbols of the rule which action triggered
@@ -2350,6 +2496,8 @@ yyreturn:
 }
 
 
+
+/* Line 1675 of yacc.c  */
 #line 495 "ast_expr2.y"
 
 
@@ -2913,18 +3061,18 @@ static struct val *op_func(struct val *funcname, struct expr_node *arglist, stru
 				}
 				
 			} else {
-				ast_log(LOG_ERROR,"Error! '%s' doesn't appear to be an available function!", funcname->u.s);
+				ast_log(LOG_ERROR, "Error! '%s' doesn't appear to be an available function!\n", funcname->u.s);
 				return (make_number ((FP___TYPE)0.0));
 			}
 #else
-			ast_log(LOG_ERROR,"Error! '%s' is not available in the standalone version!", funcname->u.s);
+			ast_log(LOG_ERROR, "Error! '%s' is not available in the standalone version!\n", funcname->u.s);
 			return (make_number ((FP___TYPE)0.0));
 #endif
 		}
 	}
 	else
 	{
-		ast_log(LOG_ERROR,"Error! '%s' is not possibly a function name!", funcname->u.s);
+		ast_log(LOG_ERROR, "Error! '%s' is not possibly a function name!\n", funcname->u.s);
 		return (make_number ((FP___TYPE)0.0));
 	}
 	return (make_number ((FP___TYPE)0.0));
@@ -3261,6 +3409,8 @@ op_compl (struct val *a)
 					v1 = 0;
 				else if (strlen(a->u.s) == 1 && a->u.s[0] == '0' )
 					v1 = 0;
+				else
+					v1 = atoi(a->u.s);
 			}
 			break;
 			
@@ -3273,6 +3423,8 @@ op_compl (struct val *a)
 					v1 = 0;
 				else if (strlen(a->u.s) == 1 && a->u.s[0] == '0' )
 					v1 = 0;
+				else
+					v1 = atoi(a->u.s);
 			}
 			break;
 		}
@@ -3408,7 +3560,7 @@ op_colon (struct val *a, struct val *b)
 	/* compile regular expression */
 	if ((eval = regcomp (&rp, b->u.s, REG_EXTENDED)) != 0) {
 		regerror (eval, &rp, errbuf, sizeof(errbuf));
-		ast_log(LOG_WARNING,"regcomp() error : %s",errbuf);
+		ast_log(LOG_WARNING, "regcomp() error : %s\n", errbuf);
 		free_value(a);
 		free_value(b);
 		return make_str("");		
@@ -3459,7 +3611,7 @@ op_eqtilde (struct val *a, struct val *b)
 	/* compile regular expression */
 	if ((eval = regcomp (&rp, b->u.s, REG_EXTENDED)) != 0) {
 		regerror (eval, &rp, errbuf, sizeof(errbuf));
-		ast_log(LOG_WARNING,"regcomp() error : %s",errbuf);
+		ast_log(LOG_WARNING, "regcomp() error : %s\n", errbuf);
 		free_value(a);
 		free_value(b);
 		return make_str("");		
