@@ -25,7 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 262801 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 313859 $")
 
 #include "asterisk/_private.h"
 #include "asterisk/paths.h"	/* use ast_config_AST_MODULE_DIR */
@@ -363,6 +363,7 @@ static char *handle_verbose(struct ast_cli_entry *e, int cmd, struct ast_cli_arg
 {
 	int oldval;
 	int newlevel;
+	unsigned int is_debug;
 	int atleast = 0;
 	int fd = a->fd;
 	int argc = a->argc;
@@ -432,21 +433,22 @@ static char *handle_verbose(struct ast_cli_entry *e, int cmd, struct ast_cli_arg
 		dst = &option_debug;
 		oldval = option_debug;
 		what = "Core debug";
+		is_debug = 1;
 	} else {
 		dst = &option_verbose;
 		oldval = option_verbose;
 		what = "Verbosity";
+		is_debug = 0;
 	}
 	if (argc == e->args + 1 && !strcasecmp(argv[e->args], "off")) {
-		unsigned int debug = (*what == 'C');
 		newlevel = 0;
 
-		dfl = debug ? &debug_files : &verbose_files;
+		dfl = is_debug ? &debug_files : &verbose_files;
 
 		AST_RWLIST_WRLOCK(dfl);
 		while ((adf = AST_RWLIST_REMOVE_HEAD(dfl, entry)))
 			ast_free(adf);
-		ast_clear_flag(&ast_options, debug ? AST_OPT_FLAG_DEBUG_FILE : AST_OPT_FLAG_VERBOSE_FILE);
+		ast_clear_flag(&ast_options, is_debug ? AST_OPT_FLAG_DEBUG_FILE : AST_OPT_FLAG_VERBOSE_FILE);
 		AST_RWLIST_UNLOCK(dfl);
 
 		goto done;
@@ -458,17 +460,23 @@ static char *handle_verbose(struct ast_cli_entry *e, int cmd, struct ast_cli_arg
 	if (sscanf(argv[e->args + atleast], "%30d", &newlevel) != 1)
 		return CLI_SHOWUSAGE;
 	if (argc == e->args + atleast + 2) {
-		unsigned int debug = (*what == 'C');
-		dfl = debug ? &debug_files : &verbose_files;
-
+		/* We have specified a module name. */
 		fn = argv[e->args + atleast + 1];
 
+		dfl = is_debug ? &debug_files : &verbose_files;
+
 		AST_RWLIST_WRLOCK(dfl);
 
-		if ((adf = find_debug_file(fn, debug)) && !newlevel) {
+		adf = find_debug_file(fn, is_debug);
+		if (!newlevel) {
+			if (!adf) {
+				/* Specified off for a nonexistent entry. */
+				AST_RWLIST_UNLOCK(dfl);
+				return CLI_SUCCESS;
+			}
 			AST_RWLIST_REMOVE(dfl, adf, entry);
 			if (AST_RWLIST_EMPTY(dfl))
-				ast_clear_flag(&ast_options, debug ? AST_OPT_FLAG_DEBUG_FILE : AST_OPT_FLAG_VERBOSE_FILE);
+				ast_clear_flag(&ast_options, is_debug ? AST_OPT_FLAG_DEBUG_FILE : AST_OPT_FLAG_VERBOSE_FILE);
 			AST_RWLIST_UNLOCK(dfl);
 			ast_cli(fd, "%s was %d and has been set to 0 for '%s'\n", what, adf->level, fn);
 			ast_free(adf);
@@ -481,23 +489,36 @@ static char *handle_verbose(struct ast_cli_entry *e, int cmd, struct ast_cli_arg
 				AST_RWLIST_UNLOCK(dfl);
 				return CLI_SUCCESS;
 			}
-		} else if (!(adf = ast_calloc(1, sizeof(*adf) + strlen(fn) + 1))) {
-			AST_RWLIST_UNLOCK(dfl);
-			return CLI_FAILURE;
+			oldval = adf->level;
+			adf->level = newlevel;
+		} else {
+			adf = ast_calloc(1, sizeof(*adf) + strlen(fn) + 1);
+			if (!adf) {
+				AST_RWLIST_UNLOCK(dfl);
+				return CLI_FAILURE;
+			}
+			oldval = adf->level;
+			adf->level = newlevel;
+			strcpy(adf->filename, fn);
+			AST_RWLIST_INSERT_TAIL(dfl, adf, entry);
 		}
 
-		oldval = adf->level;
-		adf->level = newlevel;
-		strcpy(adf->filename, fn);
+		ast_set_flag(&ast_options, is_debug ? AST_OPT_FLAG_DEBUG_FILE : AST_OPT_FLAG_VERBOSE_FILE);
 
-		ast_set_flag(&ast_options, debug ? AST_OPT_FLAG_DEBUG_FILE : AST_OPT_FLAG_VERBOSE_FILE);
-
-		AST_RWLIST_INSERT_TAIL(dfl, adf, entry);
 		AST_RWLIST_UNLOCK(dfl);
 
 		ast_cli(fd, "%s was %d and has been set to %d for '%s'\n", what, oldval, adf->level, adf->filename);
 
 		return CLI_SUCCESS;
+	} else if (!newlevel) {
+		/* Specified level as 0 instead of off. */
+		dfl = is_debug ? &debug_files : &verbose_files;
+
+		AST_RWLIST_WRLOCK(dfl);
+		while ((adf = AST_RWLIST_REMOVE_HEAD(dfl, entry)))
+			ast_free(adf);
+		ast_clear_flag(&ast_options, is_debug ? AST_OPT_FLAG_DEBUG_FILE : AST_OPT_FLAG_VERBOSE_FILE);
+		AST_RWLIST_UNLOCK(dfl);
 	}
 
 done:
@@ -1465,6 +1486,28 @@ static char *group_show_channels(struct ast_cli_entry *e, int cmd, struct ast_cl
 #undef FORMAT_STRING
 }
 
+static char *handle_cli_wait_fullybooted(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a)
+{
+	switch (cmd) {
+	case CLI_INIT:
+		e->command = "core waitfullybooted";
+		e->usage =
+			"Usage: core waitfullybooted\n"
+			"	Wait until Asterisk has fully booted.\n";
+		return NULL;
+	case CLI_GENERATE:
+		return NULL;
+	}
+
+	while (!ast_test_flag(&ast_options, AST_OPT_FLAG_FULLY_BOOTED)) {
+		usleep(100);
+	}
+
+	ast_cli(a->fd, "Asterisk has fully booted.\n");
+
+	return CLI_SUCCESS;
+}
+
 static char *handle_help(struct ast_cli_entry *e, int cmd, struct ast_cli_args *a);
 
 static struct ast_cli_entry cli_cli[] = {
@@ -1508,6 +1551,8 @@ static struct ast_cli_entry cli_cli[] = {
 	AST_CLI_DEFINE(handle_cli_show_permissions, "Show CLI permissions"),
 
 	AST_CLI_DEFINE(handle_cli_check_permissions, "Try a permissions config for a user"),
+
+	AST_CLI_DEFINE(handle_cli_wait_fullybooted, "Wait for Asterisk to be fully booted"),
 };
 
 /*!
@@ -1702,20 +1747,30 @@ static int word_match(const char *cmd, const char *cli_word)
 		return -1;
 	if (!strchr(cli_rsvd, cli_word[0])) /* normal match */
 		return (strcasecmp(cmd, cli_word) == 0) ? 1 : -1;
-	/* regexp match, takes [foo|bar] or {foo|bar} */
 	l = strlen(cmd);
 	/* wildcard match - will extend in the future */
 	if (l > 0 && cli_word[0] == '%') {
 		return 1;	/* wildcard */
 	}
+
+	/* Start a search for the command entered against the cli word in question */
 	pos = strcasestr(cli_word, cmd);
-	if (pos == NULL) /* not found, say ok if optional */
-		return cli_word[0] == '[' ? 0 : -1;
-	if (pos == cli_word)	/* no valid match at the beginning */
-		return -1;
-	if (strchr(cli_rsvd, pos[-1]) && strchr(cli_rsvd, pos[l]))
-		return 1;	/* valid match */
-	return -1;	/* not found */
+	while (pos) {
+
+		/*
+		 *Check if the word matched with is surrounded by reserved characters on both sides
+		 * and isn't at the beginning of the cli_word since that would make it check in a location we shouldn't know about.
+		 * If it is surrounded by reserved chars and isn't at the beginning, it's a match.
+		 */
+		if (pos != cli_word && strchr(cli_rsvd, pos[-1]) && strchr(cli_rsvd, pos[l])) {
+			return 1;	/* valid match */
+		}
+
+		/* Ok, that one didn't match, strcasestr to the next appearance of the command and start over.*/
+		pos = strcasestr(pos + 1, cmd);
+	}
+	/* If no matches were found over the course of the while loop, we hit the end of the string. It's a mismatch. */
+	return -1;
 }
 
 /*! \brief if word is a valid prefix for token, returns the pos-th
