@@ -25,7 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 254499 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 304096 $")
 
 #include <dirent.h>
 #include <sys/stat.h>
@@ -653,7 +653,10 @@ struct ast_filestream *ast_openstream_full(struct ast_channel *chan, const char
 	chan->oldwriteformat = chan->writeformat;
 	/* Set the channel to a format we can work with */
 	res = ast_set_write_format(chan, fmts);
- 	res = ast_filehelper(buf, chan, NULL, ACTION_OPEN);
+	if (res == -1) {	/* No format available that works with this channel */
+		return NULL;
+	}
+	res = ast_filehelper(buf, chan, NULL, ACTION_OPEN);
 	if (res >= 0)
 		return chan->stream;
 	return NULL;
@@ -958,10 +961,12 @@ int ast_streamfile(struct ast_channel *chan, const char *filename, const char *p
 	 * done this way because there is no where for ast_openstream_full to
 	 * return the file had no data. */
 	seekattempt = fseek(fs->f, -1, SEEK_END);
-	if (!seekattempt)
-		ast_seekstream(fs, 0, SEEK_SET);
-	else
+	if (seekattempt && errno == EINVAL) {
+		/* Zero-length file, as opposed to a pipe */
 		return 0;
+	} else {
+		ast_seekstream(fs, 0, SEEK_SET);
+	}
 
 	vfs = ast_openvstream(chan, filename, preflang);
 	if (vfs) {
@@ -1112,6 +1117,11 @@ struct ast_filestream *ast_writefile(const char *filename, const char *type, con
 		if (fd > -1) {
 			errno = 0;
 			fs = get_filestream(f, bfile);
+			if (fs) {
+				if ((fs->write_buffer = ast_malloc(32768))) {
+					setvbuf(fs->f, fs->write_buffer, _IOFBF, 32768);
+				}
+			}
 			if (!fs || rewrite_wrapper(fs, comment)) {
 				ast_log(LOG_WARNING, "Unable to rewrite %s\n", fn);
 				close(fd);
@@ -1138,11 +1148,6 @@ struct ast_filestream *ast_writefile(const char *filename, const char *type, con
 			}
 			fs->vfs = NULL;
 			/* If truncated, we'll be at the beginning; if not truncated, then append */
-
-			if ((fs->write_buffer = ast_malloc(32768))){
-				setvbuf(fs->f, fs->write_buffer, _IOFBF, 32768);
-			}
-
 			f->seek(fs, 0, SEEK_END);
 		} else if (errno != EEXIST) {
 			ast_log(LOG_WARNING, "Unable to open file %s: %s\n", fn, strerror(errno));
