@@ -25,7 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 243247 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 269977 $")
 
 #include "asterisk/_private.h"
 #include "asterisk/lock.h"
@@ -1394,7 +1394,9 @@ static int speex_samples(unsigned char *data, int len)
 		}
 		bit += off;
 
-		if ((len * 8 - bit) < 5) {
+		if ((len * 8 - bit) == 0) {
+			break;
+		} else if ((len * 8 - bit) < 5) {
 			ast_log(LOG_WARNING, "Not enough bits remaining after wide band for speex samples.\n");
 			break;
 		}
