@@ -29,7 +29,7 @@
 #include "asterisk.h"
 
 #if !defined(STANDALONE)
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 165792 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 310448 $")
 #endif
 
 #include <ctype.h>
@@ -55,6 +55,26 @@ static int mtx_prof = -1; /* helps the standalone compile with the mtx_prof flag
 #include "asterisk/argdesc.h"
 #endif
 
+/*** DOCUMENTATION
+	<application name="AELSub" language="en_US">
+		<synopsis>
+			Launch subroutine built with AEL
+		</synopsis>
+		<syntax>
+			<parameter name="routine" required="true">
+				<para>Named subroutine to execute.</para>
+			</parameter>
+			<parameter name="args" required="false" />
+		</syntax>
+		<description>
+			<para>Execute the named subroutine, defined in AEL, from another dialplan
+			language, such as extensions.conf, Realtime extensions, or Lua.</para>
+			<para>The purpose of this application is to provide a sane entry point into
+			AEL subroutines, the implementation of which may change from time to time.</para>
+		</description>
+	</application>
+ ***/
+
 /* these functions are in ../ast_expr2.fl */
 
 #define DEBUG_READ   (1 << 0)
@@ -95,7 +115,7 @@ struct ael_extension *new_exten(void);
 void destroy_extensions(struct ael_extension *exten);
 void set_priorities(struct ael_extension *exten);
 void add_extensions(struct ael_extension *exten);
-void ast_compile_ael2(struct ast_context **local_contexts, struct ast_hashtab *local_table, struct pval *root);
+int ast_compile_ael2(struct ast_context **local_contexts, struct ast_hashtab *local_table, struct pval *root);
 void destroy_pval(pval *item);
 void destroy_pval_item(pval *item);
 int is_float(char *arg );
@@ -108,6 +128,27 @@ static int aeldebug = 0;
 
 /* interface stuff */
 
+#ifndef STANDALONE
+static char *aelsub = "AELSub";
+
+static int aelsub_exec(struct ast_channel *chan, void *vdata)
+{
+	char buf[256], *data = ast_strdupa(vdata);
+	struct ast_app *gosub = pbx_findapp("Gosub");
+	AST_DECLARE_APP_ARGS(args,
+		AST_APP_ARG(name);
+		AST_APP_ARG(args);
+	);
+
+	if (gosub) {
+		AST_STANDARD_RAW_ARGS(args, data);
+		snprintf(buf, sizeof(buf), "%s,~~s~~,1(%s)", args.name, args.args);
+		return pbx_exec(chan, gosub, buf);
+	}
+	return -1;
+}
+#endif
+
 /* if all the below are static, who cares if they are present? */
 
 static int pbx_load_module(void)
@@ -137,7 +178,11 @@ static int pbx_load_module(void)
 	if (errs == 0 && sem_err == 0) {
 		ast_log(LOG_NOTICE, "AEL load process: checked config file name '%s'.\n", rfilename);
 		local_table = ast_hashtab_create(11, ast_hashtab_compare_contexts, ast_hashtab_resize_java, ast_hashtab_newsize_java, ast_hashtab_hash_contexts, 0);
-		ast_compile_ael2(&local_contexts, local_table, parse_tree);
+		if (ast_compile_ael2(&local_contexts, local_table, parse_tree)) {
+			ast_log(LOG_ERROR, "AEL compile failed! Aborting.\n");
+			destroy_pval(parse_tree); /* free up the memory */
+			return AST_MODULE_LOAD_DECLINE;
+		}
 		ast_log(LOG_NOTICE, "AEL load process: compiled config file name '%s'.\n", rfilename);
 		
 		ast_merge_contexts_and_delete(&local_contexts, local_table, registrar);
@@ -220,12 +265,18 @@ static int unload_module(void)
 {
 	ast_context_destroy(NULL, registrar);
 	ast_cli_unregister_multiple(cli_ael, ARRAY_LEN(cli_ael));
+#ifndef STANDALONE
+	ast_unregister_application(aelsub);
+#endif
 	return 0;
 }
 
 static int load_module(void)
 {
 	ast_cli_register_multiple(cli_ael, ARRAY_LEN(cli_ael));
+#ifndef STANDALONE
+	ast_register_application_xml(aelsub, aelsub_exec);
+#endif
 	return (pbx_load_module());
 }
 
