@@ -43,7 +43,7 @@
 #include <stdio.h>
 #include <ldap.h>
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 240280 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 304865 $")
 
 #include "asterisk/channel.h"
 #include "asterisk/logger.h"
@@ -65,7 +65,7 @@ AST_MUTEX_DEFINE_STATIC(ldap_lock);
 static LDAP *ldapConn;
 static char url[512];
 static char user[512];
-static char pass[50];
+static char pass[512];
 static char base_distinguished_name[512];
 static int version = 3;
 static time_t connect_time;
@@ -293,8 +293,6 @@ static struct ast_variable *realtime_ldap_entry_to_var(struct ldap_table_config
 				if (is_realmed_password_attribute) {
 					if (!strncasecmp(valptr, "{md5}", 5)) {
 						valptr += 5;
-					} else {
-						valptr = NULL;
 					}
 					ast_debug(2, "md5: %s\n", valptr);
 				}
@@ -332,6 +330,7 @@ static struct ast_variable *realtime_ldap_entry_to_var(struct ldap_table_config
 			}
 			ldap_value_free_len(values);
 		}
+		ldap_memfree(ldap_attribute_name);
 		ldap_attribute_name = ldap_next_attribute(ldapConn, ldap_entry, ber);
 	}
 	ber_free(ber, 0);
@@ -414,8 +413,6 @@ static struct ast_variable **realtime_ldap_result_to_vars(struct ldap_table_conf
 						if (is_realmed_password_attribute) {
 							if (strncasecmp(valptr, "{md5}", 5) == 0) {
 								valptr += 5;
-							} else {
-								valptr = NULL;
 							}
 							ast_debug(2, "md5: %s\n", valptr);
 						}
@@ -500,6 +497,7 @@ static struct ast_variable **realtime_ldap_result_to_vars(struct ldap_table_conf
 					} /*!< for (v = values; *v; v++) */
 					ldap_value_free_len(values);
 				}/*!< if (values) */
+				ldap_memfree(ldap_attribute_name);
 				ldap_attribute_name = ldap_next_attribute(ldapConn, ldap_entry, ber);
 			} /*!< while (ldap_attribute_name) */
 			ber_free(ber, 0);
@@ -938,10 +936,24 @@ static struct ast_variable *realtime_ldap(const char *basedn,
 static struct ast_config *realtime_multi_ldap(const char *basedn,
       const char *table_name, va_list ap)
 {
+        char *op;
+        const char *initfield = NULL;
+        const char *newparam, *newval;
 	struct ast_variable **vars =
 		realtime_ldap_base_ap(NULL, basedn, table_name, ap);
 	struct ast_config *cfg = NULL;
 
+        newparam = va_arg(ap, const char *);
+        newval = va_arg(ap, const char *);
+        if (!newparam || !newval) {
+            ast_log(LOG_WARNING, "realtime retrieval requires at least 1 parameter and 1 value to search on.\n");
+            return NULL;
+        }
+        initfield = ast_strdupa(newparam);
+        if ((op = strchr(initfield, ' '))) {
+                *op = '\0';
+        }
+
 	if (vars) {
 		cfg = ast_config_new();
 		if (!cfg) {
@@ -959,6 +971,9 @@ static struct ast_config *realtime_multi_ldap(const char *basedn,
 					struct ast_variable *var = *p;
 					while (var) {
 						struct ast_variable *next = var->next;
+                                                if (initfield && !strcmp(initfield, var->name)) {
+                                                        ast_category_rename(cat, var->value);
+                                                }
 						var->next = NULL;
 						ast_variable_append(cat, var);
 						var = next;
@@ -1298,7 +1313,7 @@ static int update_ldap(const char *basedn, const char *table_name, const char *a
 			dn = ldap_get_dn(ldapConn, ldap_entry);
 			if ((error = ldap_modify_ext_s(ldapConn, dn, ldap_mods, NULL, NULL)) != LDAP_SUCCESS) 
 				ast_log(LOG_ERROR, "Couldn't modify dn:%s because %s", dn, ldap_err2string(error));
-
+			ldap_memfree(dn);
 			ldap_entry = ldap_next_entry(ldapConn, ldap_entry);
 		}
 	}
@@ -1480,7 +1495,7 @@ static int update2_ldap(const char *basedn, const char *table_name, va_list ap)
 			dn = ldap_get_dn(ldapConn, ldap_entry);
 			if ((error = ldap_modify_ext_s(ldapConn, dn, ldap_mods, NULL, NULL)) != LDAP_SUCCESS) 
 				ast_log(LOG_ERROR, "Couldn't modify dn:%s because %s", dn, ldap_err2string(error));
-
+			ldap_memfree(dn);
 			ldap_entry = ldap_next_entry(ldapConn, ldap_entry);
 		}
 	}
