@@ -37,7 +37,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 209234 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 295440 $")
 
 #include <ctype.h>
 #include <iksemel.h>
@@ -618,40 +618,45 @@ static int aji_tls_handshake(struct aji_client *client)
 {
 	int ret;
 	int sock;
-	
-	ast_debug(1, "Starting TLS handshake\n"); 
+
+	ast_debug(1, "Starting TLS handshake\n");
 
 	/* Choose an SSL/TLS protocol version, create SSL_CTX */
 	client->ssl_method = SSLv3_method();
-	client->ssl_context = SSL_CTX_new(client->ssl_method);                
-	if (!client->ssl_context)
+	client->ssl_context = SSL_CTX_new((SSL_METHOD *) client->ssl_method);
+	if (!client->ssl_context) {
 		return IKS_NET_TLSFAIL;
+	}
 
 	/* Create new SSL session */
 	client->ssl_session = SSL_new(client->ssl_context);
-	if (!client->ssl_session)
+	if (!client->ssl_session) {
 		return IKS_NET_TLSFAIL;
+	}
 
 	/* Enforce TLS on our XMPP connection */
 	sock = iks_fd(client->p);
 	ret = SSL_set_fd(client->ssl_session, sock);
-	if (!ret)
+	if (!ret) {
 		return IKS_NET_TLSFAIL;
+	}
 
 	/* Perform SSL handshake */
 	ret = SSL_connect(client->ssl_session);
-	if (!ret)
+	if (!ret) {
 		return IKS_NET_TLSFAIL;
+	}
 
 	client->stream_flags &= (~TRY_SECURE);
 	client->stream_flags |= SECURE;
 
 	/* Sent over the established TLS connection */
 	ret = aji_send_header(client, client->jid->server);
-	if (ret != IKS_OK)
+	if (ret != IKS_OK) {
 		return IKS_NET_TLSFAIL;
+	}
 
-	ast_debug(1, "TLS started with server\n"); 
+	ast_debug(1, "TLS started with server\n");
 
 	return IKS_OK;
 }
@@ -668,37 +673,27 @@ static int aji_tls_handshake(struct aji_client *client)
  */
 static int aji_io_recv(struct aji_client *client, char *buffer, size_t buf_len, int timeout)
 {
-	int sock;
-	fd_set fds;
-	struct timeval tv, *tvptr = NULL;
+	struct pollfd pfd = { .events = POLLIN };
 	int len, res;
 
 #ifdef HAVE_OPENSSL
 	if (aji_is_secure(client)) {
-		sock = SSL_get_fd(client->ssl_session);
-		if (sock < 0)
-			return -1;		
+		pfd.fd = SSL_get_fd(client->ssl_session);
+		if (pfd.fd < 0) {
+			return -1;
+		}
 	} else
 #endif /* HAVE_OPENSSL */
-		sock = iks_fd(client->p);	
+		pfd.fd = iks_fd(client->p);
 
-	memset(&tv, 0, sizeof(struct timeval));
-	FD_ZERO(&fds);
-	FD_SET(sock, &fds);
-	tv.tv_sec = timeout;
-
-	/* NULL value for tvptr makes ast_select wait indefinitely */
-	tvptr = (timeout != -1) ? &tv : NULL;
-
-	/* ast_select emulates linux behaviour in terms of timeout handling */
-	res = ast_select(sock + 1, &fds, NULL, NULL, tvptr);
+	res = ast_poll(&pfd, 1, timeout > 0 ? timeout * 1000 : -1);
 	if (res > 0) {
 #ifdef HAVE_OPENSSL
 		if (aji_is_secure(client)) {
 			len = SSL_read(client->ssl_session, buffer, buf_len);
 		} else
 #endif /* HAVE_OPENSSL */
-			len = recv(sock, buffer, buf_len, 0);
+			len = recv(pfd.fd, buffer, buf_len, 0);
 
 		if (len > 0) {
 			return len;
@@ -2327,16 +2322,19 @@ static int aji_get_roster(struct aji_client *client)
 static int aji_client_connect(void *data, ikspak *pak)
 {
 	struct aji_client *client = ASTOBJ_REF((struct aji_client *) data);
-	int res = 0;
+	int res = IKS_FILTER_PASS;
 
 	if (client) {
 		if (client->state == AJI_DISCONNECTED) {
 			iks_filter_add_rule(client->f, aji_filter_roster, client, IKS_RULE_TYPE, IKS_PAK_IQ, IKS_RULE_SUBTYPE, IKS_TYPE_RESULT, IKS_RULE_ID, "roster", IKS_RULE_DONE);
 			client->state = AJI_CONNECTING;
 			client->jid = (iks_find_cdata(pak->query, "jid")) ? iks_id_new(client->stack, iks_find_cdata(pak->query, "jid")) : client->jid;
-			iks_filter_remove_hook(client->f, aji_client_connect);
-			if(!client->component) /*client*/
+			if (!client->component) { /*client*/
 				aji_get_roster(client);
+			}
+			iks_filter_remove_hook(client->f, aji_client_connect);
+			/* Once we remove the hook for this routine, we must return EAT or we will crash or corrupt memory */
+			res = IKS_FILTER_EAT;
 		}
 	} else
 		ast_log(LOG_ERROR, "Out of memory.\n");
@@ -2907,7 +2905,7 @@ static int aji_create_buddy(char *label, struct aji_client *client)
 static int aji_load_config(int reload)
 {
 	char *cat = NULL;
-	int debug = 1;
+	int debug = 0;
 	struct ast_config *cfg = NULL;
 	struct ast_variable *var = NULL;
 	struct ast_flags config_flags = { reload ? CONFIG_FLAG_FILEUNCHANGED : 0 };
@@ -3068,12 +3066,12 @@ static int unload_module(void)
 	ast_custom_function_unregister(&jabberstatus_function);
 	
 	ASTOBJ_CONTAINER_TRAVERSE(&clients, 1, {
-		ASTOBJ_RDLOCK(iterator);
+		ASTOBJ_WRLOCK(iterator);
 		ast_debug(3, "JABBER: Releasing and disconnecting client: %s\n", iterator->name);
 		iterator->state = AJI_DISCONNECTING;
-		ast_aji_disconnect(iterator);
-		pthread_join(iterator->thread, NULL);
 		ASTOBJ_UNLOCK(iterator);
+		pthread_join(iterator->thread, NULL);
+		ast_aji_disconnect(iterator);
 	});
 
 	ASTOBJ_CONTAINER_DESTROYALL(&clients, aji_client_destroy);
