@@ -37,7 +37,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 222187 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 320444 $")
 
 #include "asterisk/file.h"
 #include "asterisk/channel.h"
@@ -578,12 +578,11 @@ SQLHSTMT ast_odbc_direct_execute(struct odbc_obj *obj, SQLHSTMT (*exec_cb)(struc
 		} else if (obj->tx) {
 			ast_log(LOG_WARNING, "Failed to execute, but unable to reconnect, as we're transactional.\n");
 			break;
-		} else {
-			obj->up = 0;
-			ast_log(LOG_WARNING, "SQL Exec Direct failed.  Attempting a reconnect...\n");
-
-			odbc_obj_disconnect(obj);
-			odbc_obj_connect(obj);
+		} else if (attempt == 0) {
+			ast_log(LOG_WARNING, "SQL Execute error! Verifying connection to %s [%s]...\n", obj->parent->name, obj->parent->dsn);
+		}
+		if (!ast_odbc_sanity_check(obj)) {
+			break;
 		}
 	}
 
@@ -625,7 +624,7 @@ SQLHSTMT ast_odbc_prepare_and_execute(struct odbc_obj *obj, SQLHSTMT (*prepare_c
 					ast_log(LOG_WARNING, "SQL Execute error, but unable to reconnect, as we're transactional.\n");
 					break;
 				} else {
-					ast_log(LOG_WARNING, "SQL Execute error %d! Attempting a reconnect...\n", res);
+					ast_log(LOG_WARNING, "SQL Execute error %d! Verifying connection to %s [%s]...\n", res, obj->parent->name, obj->parent->dsn);
 					SQLFreeHandle(SQL_HANDLE_STMT, stmt);
 					stmt = NULL;
 
@@ -634,7 +633,9 @@ SQLHSTMT ast_odbc_prepare_and_execute(struct odbc_obj *obj, SQLHSTMT (*prepare_c
 					 * While this isn't the best way to try to correct an error, this won't automatically
 					 * fail when the statement handle invalidates.
 					 */
-					ast_odbc_sanity_check(obj);
+					if (!ast_odbc_sanity_check(obj)) {
+						break;
+					}
 					continue;
 				}
 			} else {
@@ -1176,10 +1177,11 @@ struct odbc_obj *ast_odbc_request_obj2(const char *name, struct ast_flags flags)
 			ast_assert(ao2_ref(obj, 0) > 1);
 		}
 
-		if (!obj && (class->count < class->limit)) {
+		if (!obj && (ast_atomic_fetchadd_int(&class->count, +1) < class->limit)) {
 			obj = ao2_alloc(sizeof(*obj), odbc_obj_destructor);
 			if (!obj) {
 				ao2_ref(class, -1);
+				ast_atomic_fetchadd_int(&class->count, -1);
 				return NULL;
 			}
 			ast_assert(ao2_ref(obj, 0) == 1);
@@ -1189,15 +1191,20 @@ struct odbc_obj *ast_odbc_request_obj2(const char *name, struct ast_flags flags)
 			class = NULL;
 			if (odbc_obj_connect(obj) == ODBC_FAIL) {
 				ast_log(LOG_WARNING, "Failed to connect to %s\n", name);
+				ast_assert(ao2_ref(obj->parent, 0) > 0);
+				/* Because it was never within the container, we have to manually decrement the count here */
+				ast_atomic_fetchadd_int(&obj->parent->count, -1);
 				ao2_ref(obj, -1);
-				ast_assert(ao2_ref(class, 0) > 0);
 				obj = NULL;
 			} else {
 				obj->used = 1;
 				ao2_link(obj->parent->obj_container, obj);
-				ast_atomic_fetchadd_int(&obj->parent->count, +1);
 			}
 		} else {
+			/* If construction fails due to the limit, remove our increment. */
+			if (!obj) {
+				ast_atomic_fetchadd_int(&class->count, -1);
+			}
 			/* Object is not constructed, so delete outstanding reference to class. */
 			ao2_ref(class, -1);
 			class = NULL;
