@@ -27,7 +27,7 @@
  ***/
 
 #include "asterisk.h"
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 201679 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 298957 $")
 
 #include "asterisk/paths.h"	/* CONFIG_DIR */
 #include <locale.h>
@@ -724,6 +724,18 @@ int ast_bt_get_addresses(struct ast_bt *bt)
 	return 0;
 }
 
+char **ast_bt_get_symbols(void **addresses, size_t num_frames)
+{
+	char **foo = calloc(num_frames, sizeof(char *) + 1);
+	if (foo) {
+		int i;
+		for (i = 0; i < num_frames; i++) {
+			foo[i] = (char *) foo + sizeof(char *) * num_frames;
+		}
+	}
+	return foo;
+}
+
 #else
 void ast_remove_lock_info(void *lock_addr)
 {
