@@ -30,7 +30,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 163991 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 298957 $")
 
 #include <pthread.h>
 #include <sys/stat.h>
@@ -395,6 +395,18 @@ int ast_bt_get_addresses(struct ast_bt *bt)
 	return -1;
 }
 
+char **ast_bt_get_symbols(void **addresses, size_t num_frames)
+{
+	char **foo = calloc(num_frames, sizeof(char *) + 1);
+	if (foo) {
+		int i;
+		for (i = 0; i < num_frames; i++) {
+			foo[i] = (char *) foo + sizeof(char *) * num_frames;
+		}
+	}
+	return foo;
+}
+
 void *ast_bt_destroy(struct ast_bt *bt)
 {
 	return NULL;
